<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://codeboxr.com
 * @since      1.0.0
 *
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    CBXTakeaTour
 * @subpackage CBXTakeaTour/includes
 * @author     Codeboxr Team <sabuj@codeboxr.com>
 */
class CBXTakeaTour {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      CBXTakeaTour_Loader $loader Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $plugin_name The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string $version The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = CBXTAKEATOUR_PLUGIN_NAME;
		$this->version     = CBXTAKEATOUR_PLUGIN_VERSION;

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - CBXTakeaTour_Loader. Orchestrates the hooks of the plugin.
	 * - CBXTakeaTour_i18n. Defines internationalization functionality.
	 * - CBXTakeaTour_Admin. Defines all hooks for the admin area.
	 * - CBXTakeaTour_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbxtakeatour-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbxtakeatour-i18n.php';

		require plugin_dir_path( dirname( __FILE__ ) ) . 'includes/cbxtakeatour-tpl-loader.php';

		// Loading Settings Class
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbxtakeatour-settings.php';

		/**
		 * This class handles all the helper method for CBX Take A Tour
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-cbxtakeatour-helper.php';


		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-cbxtakeatour-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-cbxtakeatour-public.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/cbxtakeatour-functions.php';

		$this->loader = new CBXTakeaTour_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the CBXTakeaTour_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new CBXTakeaTour_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new CBXTakeaTour_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_filter( 'default_title', $plugin_admin, 'default_title_extend' );
		$this->loader->add_filter( 'months_dropdown_results', $plugin_admin, 'months_dropdown_results_extend', 10, 2 );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_admin, 'create_tour', 0 );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'setting_init' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'plugin_reset', 1 );
		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'add_meta_boxes' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_post', 10, 2 );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'admin_pages' );

		$this->loader->add_filter( 'manage_edit-cbxtour_columns', $plugin_admin, 'add_new_tour_columns' );
		$this->loader->add_action( 'manage_cbxtour_posts_custom_column', $plugin_admin, 'manage_tour_columns' );


		//plugin setting link, upgrade process, admin notices
		$this->loader->add_filter( 'plugin_action_links_' . CBXTAKEATOUR_BASE_NAME, $plugin_admin, 'plugin_action_links' );
		$this->loader->add_filter( 'plugin_row_meta', $plugin_admin, 'plugin_row_meta', 10, 4 );
		$this->loader->add_action( 'upgrader_process_complete', $plugin_admin, 'plugin_upgrader_process_complete', 10, 2 );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'plugin_activate_upgrade_notices' );

		//single tour button restrictions related
		//remove admin menu add new feedback button
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'remove_menus' );
		$this->loader->add_action( 'cbxtour_remove', $plugin_admin, 'cbxtour_remove_core', 10 );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'cbxtour_notice' );

		//gutenberg
		$this->loader->add_action( 'init', $plugin_admin, 'gutenberg_blocks' );
		$this->loader->add_filter( 'block_categories', $plugin_admin, 'gutenberg_block_categories', 10, 2 );
		$this->loader->add_action( 'enqueue_block_editor_assets', $plugin_admin, 'enqueue_block_editor_assets' );

		//update manager
		$this->loader->add_filter('pre_set_site_transient_update_plugins', $plugin_admin, 'pre_set_site_transient_update_plugins_pro_addon');
		$this->loader->add_action( 'in_plugin_update_message-' . 'cbxtakeatourpro/cbxtakeatourpro.php', $plugin_admin, 'plugin_update_message_pro_addons' );
	}//end define_admin_hooks

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {
		$plugin_public = new CBXTakeaTour_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		$this->loader->add_action( 'init', $plugin_public, 'init_shortode' );

		//Classic Widget
		$this->loader->add_action( 'widgets_init', $plugin_public, 'register_widget' );

		//Elementor Widget
		$this->loader->add_action( 'elementor/widgets/widgets_registered', $plugin_public, 'init_elementor_widgets' );
		$this->loader->add_action( 'elementor/elements/categories_registered', $plugin_public, 'add_elementor_widget_categories' );
		$this->loader->add_action( 'elementor/editor/before_enqueue_scripts', $plugin_public, 'elementor_icon_loader', 99999 );

		//visual composer widget
		$this->loader->add_action( 'vc_before_init', $plugin_public, 'vc_before_init_actions' );
	}//end define_public_hooks

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 * @since     1.0.0
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @return    CBXTakeaTour_Loader    Orchestrates the hooks of the plugin.
	 * @since     1.0.0
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 * @since     1.0.0
	 */
	public function get_version() {
		return $this->version;
	}

}//end class CBXTakeaTour
